@@ -366,7 +366,7 @@ class Provider:
 
         self.validate_package_for_dependency(dependency=dependency, package=package)
 
-        for extra in dependency.extras:
+        for extra in sorted(dependency.extras):
             if extra in package.extras:
                 for dep in package.extras[extra]:
                     dep.activate()
@@ -487,7 +487,7 @@ class Provider:
         if dependency.extras:
             # Find all the optional dependencies that are wanted - taking care to allow
             # for self-referential extras.
-            stack = list(dependency.extras)
+            stack = sorted(dependency.extras)
             while stack:
                 extra = stack.pop()
                 if extra in found_extras:
@@ -497,16 +497,14 @@ class Provider:
                 extra_dependencies = package.extras.get(extra, [])
                 for extra_dependency in extra_dependencies:
                     if extra_dependency.name == dependency.name:
-                        stack += list(extra_dependency.extras)
+                        stack += sorted(extra_dependency.extras)
                     else:
                         optional_dependencies.add(extra_dependency.name)
 
             # If some extras/features were required, we need to add a special dependency
             # representing the base package to the current package.
 
-            dependency_package = dependency_package.with_features(
-                list(dependency.extras)
-            )
+            dependency_package = dependency_package.with_features(dependency.extras)
             package = dependency_package.package
             dependency = dependency_package.dependency
             new_dependency = package.without_features().to_dependency()
