@@ -1,6 +1,8 @@
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 from distutils.core import setup
 
+
 packages = ["project_with_extras"]
 
 package_data = {"": ["*"]}
