@@ -1,6 +1,8 @@
 from __future__ import annotations
 
+import contextlib
 import shutil
+import uuid
 
 from subprocess import CalledProcessError
 from typing import TYPE_CHECKING
@@ -8,26 +10,25 @@ from zipfile import ZipFile
 
 import pytest
 
+from build import BuildBackendException
+from build import ProjectBuilder
 from packaging.metadata import parse_email
+from pkginfo.distribution import NewMetadataVersion
 
 from poetry.inspection.info import PackageInfo
 from poetry.inspection.info import PackageInfoError
-from poetry.utils.env import EnvCommandError
 from poetry.utils.env import VirtualEnv
 
 
 if TYPE_CHECKING:
+    from collections.abc import Iterator
     from pathlib import Path
 
     from packaging.metadata import RawMetadata
     from pytest_mock import MockerFixture
 
     from tests.types import FixtureDirGetter
-
-
-@pytest.fixture(autouse=True)
-def pep517_metadata_mock() -> None:
-    pass
+    from tests.types import SetProjectContext
 
 
 @pytest.fixture
@@ -61,7 +62,8 @@ def demo_setup(source_dir: Path) -> Path:
         "from setuptools import setup; "
         'setup(name="demo", '
         'version="0.1.0", '
-        'install_requires=["package"])'
+        'install_requires=["package"])',
+        encoding="utf-8",
     )
     return source_dir
 
@@ -78,7 +80,8 @@ def demo_setup_cfg(source_dir: Path) -> Path:
                 "[options]",
                 "install_requires = package",
             ]
-        )
+        ),
+        encoding="utf-8",
     )
     return source_dir
 
@@ -90,7 +93,8 @@ def demo_setup_complex(source_dir: Path) -> Path:
         "from setuptools import setup; "
         'setup(name="demo", '
         'version="0.1.0", '
-        'install_requires=[i for i in ["package"]])'
+        'install_requires=[i for i in ["package"]])',
+        encoding="utf-8",
     )
     return source_dir
 
@@ -98,7 +102,9 @@ def demo_setup_complex(source_dir: Path) -> Path:
 @pytest.fixture
 def demo_setup_complex_pep517_legacy(demo_setup_complex: Path) -> Path:
     pyproject_toml = demo_setup_complex / "pyproject.toml"
-    pyproject_toml.write_text('[build-system]\nrequires = ["setuptools", "wheel"]')
+    pyproject_toml.write_text(
+        '[build-system]\nrequires = ["setuptools", "wheel"]', encoding="utf-8"
+    )
     return demo_setup_complex
 
 
@@ -116,7 +122,8 @@ def demo_setup_complex_calls_script(
     [build-system]
     requires = ["setuptools", "scripts @ {scripts_dir.as_uri()}"]
     build-backend = "setuptools.build_meta:__legacy__"
-"""
+""",
+        encoding="utf-8",
     )
 
     setup_py = source_dir / "setup.py"
@@ -127,12 +134,19 @@ from setuptools import setup
 if subprocess.call(["exit-code"]) != 42:
     raise RuntimeError("Wrong exit code.")
 setup(name="demo", version="0.1.0", install_requires=[i for i in ["package"]])
-"""
+""",
+        encoding="utf-8",
     )
 
     return source_dir
 
 
+@pytest.fixture(autouse=True)
+def use_project_context(set_project_context: SetProjectContext) -> Iterator[None]:
+    with set_project_context("sample_project"):
+        yield
+
+
 def demo_check_info(info: PackageInfo, requires_dist: set[str] | None = None) -> None:
     assert info.name == "demo"
     assert info.version == "0.1.0"
@@ -141,19 +155,29 @@ def demo_check_info(info: PackageInfo, requires_dist: set[str] | None = None) ->
     if requires_dist:
         assert set(info.requires_dist) == requires_dist
     else:
+        # Exact formatting various according to the exact mechanism used to extract the
+        # metadata.
         assert set(info.requires_dist) in (
-            # before https://github.com/python-poetry/poetry-core/pull/510
             {
                 'cleo; extra == "foo"',
                 "pendulum (>=1.4.4)",
                 'tomlkit; extra == "bar"',
             },
-            # after https://github.com/python-poetry/poetry-core/pull/510
             {
                 'cleo ; extra == "foo"',
                 "pendulum (>=1.4.4)",
                 'tomlkit ; extra == "bar"',
             },
+            {
+                'cleo ; extra == "foo"',
+                "pendulum>=1.4.4",
+                'tomlkit ; extra == "bar"',
+            },
+            {
+                "cleo ; extra == 'foo'",
+                "pendulum (>=1.4.4)",
+                "tomlkit ; extra == 'bar'",
+            },
         )
 
 
@@ -179,6 +203,25 @@ def test_info_from_wheel(demo_wheel: Path) -> None:
     assert info._source_url == demo_wheel.resolve().as_posix()
 
 
+@pytest.mark.parametrize("version", ["23", "24", "299"])
+def test_info_from_wheel_metadata_versions(
+    version: str, fixture_dir: FixtureDirGetter
+) -> None:
+    path = (
+        fixture_dir("distributions")
+        / f"demo_metadata_version_{version}-0.1.0-py2.py3-none-any.whl"
+    )
+    with (
+        pytest.warns(NewMetadataVersion)
+        if version == "299"
+        else contextlib.nullcontext()
+    ):
+        info = PackageInfo.from_wheel(path)
+    demo_check_info(info)
+    assert info._source_type == "file"
+    assert info._source_url == path.resolve().as_posix()
+
+
 def test_info_from_wheel_metadata_version_unknown(
     fixture_dir: FixtureDirGetter,
 ) -> None:
@@ -187,7 +230,7 @@ def test_info_from_wheel_metadata_version_unknown(
         / "demo_metadata_version_unknown-0.1.0-py2.py3-none-any.whl"
     )
 
-    with pytest.raises(PackageInfoError) as e:
+    with pytest.warns(NewMetadataVersion), pytest.raises(PackageInfoError) as e:
         PackageInfo.from_wheel(path)
 
     assert "Unknown metadata version: 999.3" in str(e.value)
@@ -223,9 +266,7 @@ def test_info_from_bdist(demo_wheel: Path) -> None:
 
 
 def test_info_from_poetry_directory(fixture_dir: FixtureDirGetter) -> None:
-    info = PackageInfo.from_directory(
-        fixture_dir("inspection") / "demo", disable_build=True
-    )
+    info = PackageInfo.from_directory(fixture_dir("inspection") / "demo")
     demo_check_info(info)
 
 
@@ -255,21 +296,31 @@ def test_info_from_requires_txt(fixture_dir: FixtureDirGetter) -> None:
     demo_check_info(info)
 
 
-def test_info_from_setup_py(demo_setup: Path) -> None:
-    info = PackageInfo.from_setup_files(demo_setup)
-    demo_check_info(info, requires_dist={"package"})
-
-
-def test_info_from_setup_cfg(demo_setup_cfg: Path) -> None:
-    info = PackageInfo.from_setup_files(demo_setup_cfg)
-    demo_check_info(info, requires_dist={"package"})
-
-
 def test_info_no_setup_pkg_info_no_deps(fixture_dir: FixtureDirGetter) -> None:
-    info = PackageInfo.from_directory(
-        fixture_dir("inspection") / "demo_no_setup_pkg_info_no_deps",
-        disable_build=True,
+    info = PackageInfo.from_metadata_directory(
+        fixture_dir("inspection") / "demo_no_setup_pkg_info_no_deps"
     )
+    assert info is not None
+    assert info.name == "demo"
+    assert info.version == "0.1.0"
+    assert info.requires_dist is None
+
+
+def test_info_no_setup_pkg_info_no_deps_for_sure(fixture_dir: FixtureDirGetter) -> None:
+    info = PackageInfo.from_metadata_directory(
+        fixture_dir("inspection") / "demo_no_setup_pkg_info_no_deps_for_sure",
+    )
+    assert info is not None
+    assert info.name == "demo"
+    assert info.version == "0.1.0"
+    assert info.requires_dist == []
+
+
+def test_info_no_setup_pkg_info_no_deps_dynamic(fixture_dir: FixtureDirGetter) -> None:
+    info = PackageInfo.from_metadata_directory(
+        fixture_dir("inspection") / "demo_no_setup_pkg_info_no_deps_dynamic",
+    )
+    assert info is not None
     assert info.name == "demo"
     assert info.version == "0.1.0"
     assert info.requires_dist is None
@@ -278,18 +329,11 @@ def test_info_no_setup_pkg_info_no_deps(fixture_dir: FixtureDirGetter) -> None:
 def test_info_setup_simple(mocker: MockerFixture, demo_setup: Path) -> None:
     spy = mocker.spy(VirtualEnv, "run")
     info = PackageInfo.from_directory(demo_setup)
-    assert spy.call_count == 0
+
+    assert spy.call_count == 6
     demo_check_info(info, requires_dist={"package"})
 
 
-def test_info_setup_cfg(mocker: MockerFixture, demo_setup_cfg: Path) -> None:
-    spy = mocker.spy(VirtualEnv, "run")
-    info = PackageInfo.from_directory(demo_setup_cfg)
-    assert spy.call_count == 0
-    demo_check_info(info, requires_dist={"package"})
-
-
-@pytest.mark.network
 def test_info_setup_complex(demo_setup_complex: Path) -> None:
     info = PackageInfo.from_directory(demo_setup_complex)
     demo_check_info(info, requires_dist={"package"})
@@ -298,17 +342,25 @@ def test_info_setup_complex(demo_setup_complex: Path) -> None:
 def test_info_setup_complex_pep517_error(
     mocker: MockerFixture, demo_setup_complex: Path
 ) -> None:
+    output = uuid.uuid4().hex
     mocker.patch(
-        "poetry.utils.env.VirtualEnv.run",
+        "build.ProjectBuilder.from_isolated_env",
         autospec=True,
-        side_effect=EnvCommandError(CalledProcessError(1, "mock", output="mock")),
+        side_effect=BuildBackendException(CalledProcessError(1, "mock", output=output)),
     )
 
-    with pytest.raises(PackageInfoError):
+    with pytest.raises(PackageInfoError) as exc:
         PackageInfo.from_directory(demo_setup_complex)
 
+    text = str(exc.value)
+    assert "Command 'mock' returned non-zero exit status 1." in text
+    assert output in text
+    assert (
+        "This error originates from the build backend, and is likely not a problem with poetry"
+        in text
+    )
+
 
-@pytest.mark.network
 def test_info_setup_complex_pep517_legacy(
     demo_setup_complex_pep517_legacy: Path,
 ) -> None:
@@ -316,23 +368,13 @@ def test_info_setup_complex_pep517_legacy(
     demo_check_info(info, requires_dist={"package"})
 
 
-def test_info_setup_complex_disable_build(
-    mocker: MockerFixture, demo_setup_complex: Path
-) -> None:
-    # Cannot extract install_requires from list comprehension.
-    with pytest.raises(PackageInfoError):
-        PackageInfo.from_directory(demo_setup_complex, disable_build=True)
-
-
-@pytest.mark.network
 def test_info_setup_complex_calls_script(demo_setup_complex_calls_script: Path) -> None:
     """Building the project requires calling a script from its build_requires."""
     info = PackageInfo.from_directory(demo_setup_complex_calls_script)
     demo_check_info(info, requires_dist={"package"})
 
 
-@pytest.mark.network
-@pytest.mark.parametrize("missing", ["version", "name", "install_requires"])
+@pytest.mark.parametrize("missing", ["version", "name"])
 def test_info_setup_missing_mandatory_should_trigger_pep517(
     mocker: MockerFixture, source_dir: Path, missing: str
 ) -> None:
@@ -340,13 +382,13 @@ def test_info_setup_missing_mandatory_should_trigger_pep517(
     setup += "setup("
     setup += 'name="demo", ' if missing != "name" else ""
     setup += 'version="0.1.0", ' if missing != "version" else ""
-    setup += 'install_requires=["package"]' if missing != "install_requires" else ""
+    setup += 'install_requires=["package"]'
     setup += ")"
 
     setup_py = source_dir / "setup.py"
-    setup_py.write_text(setup)
+    setup_py.write_text(setup, encoding="utf-8")
 
-    spy = mocker.spy(VirtualEnv, "run")
+    spy = mocker.spy(ProjectBuilder, "from_isolated_env")
     _ = PackageInfo.from_directory(source_dir)
     assert spy.call_count == 1
 
