@@ -1,30 +1,24 @@
 from __future__ import annotations
 
-import re
-
 from pathlib import Path
 from typing import TYPE_CHECKING
-from typing import Any
 
 import pytest
 
 from packaging.tags import Tag
 from poetry.core.packages.package import Package
 
+from poetry.console.exceptions import PoetryRuntimeError
 from poetry.installation.chooser import Chooser
 from poetry.repositories.legacy_repository import LegacyRepository
-from poetry.repositories.pypi_repository import PyPiRepository
-from poetry.repositories.repository_pool import RepositoryPool
 from poetry.utils.env import MockEnv
 
 
 if TYPE_CHECKING:
-    import httpretty
-
-    from httpretty.core import HTTPrettyRequest
-
+    from poetry.repositories.repository_pool import RepositoryPool
     from tests.conftest import Config
-
+    from tests.types import DistributionHashGetter
+    from tests.types import SpecializedLegacyRepositoryMocker
 
 JSON_FIXTURES = (
     Path(__file__).parent.parent / "repositories" / "fixtures" / "pypi.org" / "json"
@@ -33,121 +27,50 @@ JSON_FIXTURES = (
 LEGACY_FIXTURES = Path(__file__).parent.parent / "repositories" / "fixtures" / "legacy"
 
 
-@pytest.fixture()
-def env() -> MockEnv:
-    return MockEnv(
-        supported_tags=[
-            Tag("cp37", "cp37", "macosx_10_15_x86_64"),
-            Tag("py3", "none", "any"),
-        ]
-    )
-
-
-@pytest.fixture()
-def mock_pypi(http: type[httpretty.httpretty]) -> None:
-    def callback(
-        request: HTTPrettyRequest, uri: str, headers: dict[str, Any]
-    ) -> list[int | dict[str, Any] | str] | None:
-        parts = uri.rsplit("/")
-
-        name = parts[-3]
-        version = parts[-2]
-
-        fixture = JSON_FIXTURES / name / (version + ".json")
-        if not fixture.exists():
-            fixture = JSON_FIXTURES / (name + ".json")
-
-        if not fixture.exists():
-            return None
-
-        with fixture.open(encoding="utf-8") as f:
-            return [200, headers, f.read()]
-
-    http.register_uri(
-        http.GET,
-        re.compile("^https://pypi.org/(.+?)/(.+?)/json$"),
-        body=callback,
-    )
-
-
-@pytest.fixture()
-def mock_legacy(http: type[httpretty.httpretty]) -> None:
-    def callback(
-        request: HTTPrettyRequest, uri: str, headers: dict[str, Any]
-    ) -> list[int | dict[str, Any] | str]:
-        parts = uri.rsplit("/")
-        name = parts[-2]
-
-        fixture = LEGACY_FIXTURES / (name + ".html")
-
-        with fixture.open(encoding="utf-8") as f:
-            return [200, headers, f.read()]
-
-    http.register_uri(
-        http.GET,
-        re.compile("^https://foo.bar/simple/(.+?)$"),
-        body=callback,
-    )
-
-
-@pytest.fixture()
-def mock_legacy_partial_yank(http: type[httpretty.httpretty]) -> None:
-    def callback(
-        request: HTTPrettyRequest, uri: str, headers: dict[str, Any]
-    ) -> list[int | dict[str, Any] | str]:
-        parts = uri.rsplit("/")
-        name = parts[-2]
-
-        fixture = LEGACY_FIXTURES / (name + "-partial-yank" + ".html")
-
-        with fixture.open(encoding="utf-8") as f:
-            return [200, headers, f.read()]
-
-    http.register_uri(
-        http.GET,
-        re.compile("^https://foo2.bar/simple/(.+?)$"),
-        body=callback,
-    )
-
-
-@pytest.fixture()
-def pool() -> RepositoryPool:
-    pool = RepositoryPool()
-
-    pool.add_repository(PyPiRepository(disable_cache=True))
-    pool.add_repository(
-        LegacyRepository("foo", "https://foo.bar/simple/", disable_cache=True)
-    )
-    pool.add_repository(
-        LegacyRepository("foo2", "https://foo2.bar/simple/", disable_cache=True)
-    )
-
-    return pool
-
-
-@pytest.mark.parametrize("source_type", ["", "legacy"])
-def test_chooser_chooses_universal_wheel_link_if_available(
+def check_chosen_link_filename(
     env: MockEnv,
-    mock_pypi: None,
-    mock_legacy: None,
     source_type: str,
     pool: RepositoryPool,
+    filename: str | None,
+    config: Config | None = None,
+    package_name: str = "pytest",
+    package_version: str = "3.5.0",
 ) -> None:
-    chooser = Chooser(pool, env)
+    chooser = Chooser(pool, env, config)
+    package = Package(package_name, package_version)
 
-    package = Package("pytest", "3.5.0")
     if source_type == "legacy":
         package = Package(
             package.name,
             package.version.text,
             source_type="legacy",
             source_reference="foo",
-            source_url="https://foo.bar/simple/",
+            source_url="https://legacy.foo.bar/simple/",
         )
 
-    link = chooser.choose_for(package)
+    try:
+        link = chooser.choose_for(package)
+    except PoetryRuntimeError as e:
+        if filename is None:
+            assert (
+                str(e)
+                == f"Unable to find installation candidates for {package.name} ({package.version})"
+            )
+        else:
+            pytest.fail("Package was not found")
+    else:
+        assert link.filename == filename
 
-    assert link.filename == "pytest-3.5.0-py2.py3-none-any.whl"
+
+@pytest.mark.parametrize("source_type", ["", "legacy"])
+def test_chooser_chooses_universal_wheel_link_if_available(
+    env: MockEnv,
+    source_type: str,
+    pool: RepositoryPool,
+) -> None:
+    check_chosen_link_filename(
+        env, source_type, pool, "pytest-3.5.0-py2.py3-none-any.whl"
+    )
 
 
 @pytest.mark.parametrize(
@@ -163,8 +86,6 @@ def test_chooser_chooses_universal_wheel_link_if_available(
 @pytest.mark.parametrize("source_type", ["", "legacy"])
 def test_chooser_no_binary_policy(
     env: MockEnv,
-    mock_pypi: None,
-    mock_legacy: None,
     source_type: str,
     pool: RepositoryPool,
     policy: str,
@@ -172,116 +93,125 @@ def test_chooser_no_binary_policy(
     config: Config,
 ) -> None:
     config.merge({"installer": {"no-binary": policy.split(",")}})
+    check_chosen_link_filename(env, source_type, pool, filename, config)
 
-    chooser = Chooser(pool, env, config)
 
-    package = Package("pytest", "3.5.0")
-    if source_type == "legacy":
-        package = Package(
-            package.name,
-            package.version.text,
-            source_type="legacy",
-            source_reference="foo",
-            source_url="https://foo.bar/simple/",
-        )
+@pytest.mark.parametrize(
+    ("policy", "filename"),
+    [
+        (":all:", "pytest-3.5.0-py2.py3-none-any.whl"),
+        (":none:", "pytest-3.5.0-py2.py3-none-any.whl"),
+        ("black", "pytest-3.5.0-py2.py3-none-any.whl"),
+        ("pytest", "pytest-3.5.0-py2.py3-none-any.whl"),
+        ("pytest,black", "pytest-3.5.0-py2.py3-none-any.whl"),
+    ],
+)
+@pytest.mark.parametrize("source_type", ["", "legacy"])
+def test_chooser_only_binary_policy(
+    env: MockEnv,
+    source_type: str,
+    pool: RepositoryPool,
+    policy: str,
+    filename: str,
+    config: Config,
+) -> None:
+    config.merge({"installer": {"only-binary": policy.split(",")}})
+    check_chosen_link_filename(env, source_type, pool, filename, config)
 
-    link = chooser.choose_for(package)
 
-    assert link.filename == filename
+@pytest.mark.parametrize(
+    ("no_binary", "only_binary", "filename"),
+    [
+        (":all:", ":all:", None),
+        (":none:", ":none:", "pytest-3.5.0-py2.py3-none-any.whl"),
+        (":none:", ":all:", "pytest-3.5.0-py2.py3-none-any.whl"),
+        (":all:", ":none:", "pytest-3.5.0.tar.gz"),
+        ("black", "black", "pytest-3.5.0-py2.py3-none-any.whl"),
+        ("black", "pytest", "pytest-3.5.0-py2.py3-none-any.whl"),
+        ("pytest", "black", "pytest-3.5.0.tar.gz"),
+        ("pytest", "pytest", None),
+        ("pytest,black", "pytest,black", None),
+    ],
+)
+@pytest.mark.parametrize("source_type", ["", "legacy"])
+def test_chooser_multiple_binary_policy(
+    env: MockEnv,
+    source_type: str,
+    pool: RepositoryPool,
+    no_binary: str,
+    only_binary: str,
+    filename: str | None,
+    config: Config,
+) -> None:
+    config.merge(
+        {
+            "installer": {
+                "no-binary": no_binary.split(","),
+                "only-binary": only_binary.split(","),
+            }
+        }
+    )
+    check_chosen_link_filename(env, source_type, pool, filename, config)
 
 
 @pytest.mark.parametrize("source_type", ["", "legacy"])
 def test_chooser_chooses_specific_python_universal_wheel_link_if_available(
     env: MockEnv,
-    mock_pypi: None,
-    mock_legacy: None,
     source_type: str,
     pool: RepositoryPool,
 ) -> None:
-    chooser = Chooser(pool, env)
-
-    package = Package("isort", "4.3.4")
-    if source_type == "legacy":
-        package = Package(
-            package.name,
-            package.version.text,
-            source_type="legacy",
-            source_reference="foo",
-            source_url="https://foo.bar/simple/",
-        )
-
-    link = chooser.choose_for(package)
-
-    assert link.filename == "isort-4.3.4-py3-none-any.whl"
+    check_chosen_link_filename(
+        env, source_type, pool, "isort-4.3.4-py3-none-any.whl", None, "isort", "4.3.4"
+    )
 
 
 @pytest.mark.parametrize("source_type", ["", "legacy"])
 def test_chooser_chooses_system_specific_wheel_link_if_available(
-    mock_pypi: None, mock_legacy: None, source_type: str, pool: RepositoryPool
+    source_type: str, pool: RepositoryPool
 ) -> None:
     env = MockEnv(
         supported_tags=[Tag("cp37", "cp37m", "win32"), Tag("py3", "none", "any")]
     )
-    chooser = Chooser(pool, env)
-
-    package = Package("pyyaml", "3.13.0")
-    if source_type == "legacy":
-        package = Package(
-            package.name,
-            package.version.text,
-            source_type="legacy",
-            source_reference="foo",
-            source_url="https://foo.bar/simple/",
-        )
-
-    link = chooser.choose_for(package)
-
-    assert link.filename == "PyYAML-3.13-cp37-cp37m-win32.whl"
+    check_chosen_link_filename(
+        env,
+        source_type,
+        pool,
+        "PyYAML-3.13-cp37-cp37m-win32.whl",
+        None,
+        "pyyaml",
+        "3.13.0",
+    )
 
 
 @pytest.mark.parametrize("source_type", ["", "legacy"])
 def test_chooser_chooses_sdist_if_no_compatible_wheel_link_is_available(
     env: MockEnv,
-    mock_pypi: None,
-    mock_legacy: None,
     source_type: str,
     pool: RepositoryPool,
 ) -> None:
-    chooser = Chooser(pool, env)
-
-    package = Package("pyyaml", "3.13.0")
-    if source_type == "legacy":
-        package = Package(
-            package.name,
-            package.version.text,
-            source_type="legacy",
-            source_reference="foo",
-            source_url="https://foo.bar/simple/",
-        )
-
-    link = chooser.choose_for(package)
-
-    assert link.filename == "PyYAML-3.13.tar.gz"
+    check_chosen_link_filename(
+        env, source_type, pool, "PyYAML-3.13.tar.gz", None, "pyyaml", "3.13.0"
+    )
 
 
 @pytest.mark.parametrize("source_type", ["", "legacy"])
 def test_chooser_chooses_distributions_that_match_the_package_hashes(
     env: MockEnv,
-    mock_pypi: None,
-    mock_legacy: None,
     source_type: str,
     pool: RepositoryPool,
+    dist_hash_getter: DistributionHashGetter,
 ) -> None:
     chooser = Chooser(pool, env)
 
     package = Package("isort", "4.3.4")
     files = [
         {
-            "hash": (
-                "sha256:b9c40e9750f3d77e6e4d441d8b0266cf555e7cdabdcff33c4fd06366ca761ef8"
-            ),
-            "filename": "isort-4.3.4.tar.gz",
+            "file": filename,
+            "hash": (f"sha256:{dist_hash_getter(filename).sha256}"),
         }
+        for filename in [
+            f"{package.name}-{package.version}.tar.gz",
+        ]
     ]
     if source_type == "legacy":
         package = Package(
@@ -289,7 +219,7 @@ def test_chooser_chooses_distributions_that_match_the_package_hashes(
             package.version.text,
             source_type="legacy",
             source_reference="foo",
-            source_url="https://foo.bar/simple/",
+            source_url="https://legacy.foo.bar/simple/",
         )
 
     package.files = files
@@ -302,21 +232,19 @@ def test_chooser_chooses_distributions_that_match_the_package_hashes(
 @pytest.mark.parametrize("source_type", ["", "legacy"])
 def test_chooser_chooses_yanked_if_no_others(
     env: MockEnv,
-    mock_pypi: None,
-    mock_legacy: None,
     source_type: str,
     pool: RepositoryPool,
+    dist_hash_getter: DistributionHashGetter,
 ) -> None:
     chooser = Chooser(pool, env)
 
     package = Package("black", "21.11b0")
     files = [
         {
-            "filename": "black-21.11b0-py3-none-any.whl",
-            "hash": (
-                "sha256:0b1f66cbfadcd332ceeaeecf6373d9991d451868d2e2219ad0ac1213fb701117"
-            ),
+            "filename": filename,
+            "hash": (f"sha256:{dist_hash_getter(filename).sha256}"),
         }
+        for filename in [f"{package.name}-{package.version}-py3-none-any.whl"]
     ]
     if source_type == "legacy":
         package = Package(
@@ -324,7 +252,7 @@ def test_chooser_chooses_yanked_if_no_others(
             package.version.text,
             source_type="legacy",
             source_reference="foo",
-            source_url="https://foo.bar/simple/",
+            source_url="https://legacy.foo.bar/simple/",
         )
 
     package.files = files
@@ -336,36 +264,44 @@ def test_chooser_chooses_yanked_if_no_others(
 
 
 def test_chooser_does_not_choose_yanked_if_others(
-    mock_legacy: None,
-    mock_legacy_partial_yank: None,
+    specialized_legacy_repository_mocker: SpecializedLegacyRepositoryMocker,
     pool: RepositoryPool,
+    dist_hash_getter: DistributionHashGetter,
 ) -> None:
     chooser = Chooser(pool, MockEnv(supported_tags=[Tag("py2", "none", "any")]))
 
+    repo = pool.repository("foo2")
+    pool.remove_repository("foo2")
+
+    assert isinstance(repo, LegacyRepository)
+    pool.add_repository(
+        specialized_legacy_repository_mocker("-partial-yank", repo.name, repo.url)
+    )
+
     package = Package("futures", "3.2.0")
     files = [
         {
-            "filename": "futures-3.2.0-py2-none-any.whl",
-            "hash": "sha256:ec0a6cb848cc212002b9828c3e34c675e0c9ff6741dc445cab6fdd4e1085d1f1",
-        },
-        {
-            "filename": "futures-3.2.0.tar.gz",
-            "hash": "sha256:9ec02aa7d674acb8618afb127e27fde7fc68994c0437ad759fa094a574adb265",
-        },
+            "filename": filename,
+            "hash": (f"sha256:{dist_hash_getter(filename).sha256}"),
+        }
+        for filename in [
+            f"{package.name}-{package.version}-py2-none-any.whl",
+            f"{package.name}-{package.version}.tar.gz",
+        ]
     ]
     package = Package(
         package.name,
         package.version.text,
         source_type="legacy",
         source_reference="foo",
-        source_url="https://foo.bar/simple/",
+        source_url="https://legacy.foo.bar/simple/",
     )
     package_partial_yank = Package(
         package.name,
         package.version.text,
         source_type="legacy",
         source_reference="foo2",
-        source_url="https://foo2.bar/simple/",
+        source_url="https://legacy.foo2.bar/simple/",
     )
 
     package.files = files
@@ -381,8 +317,6 @@ def test_chooser_does_not_choose_yanked_if_others(
 @pytest.mark.parametrize("source_type", ["", "legacy"])
 def test_chooser_throws_an_error_if_package_hashes_do_not_match(
     env: MockEnv,
-    mock_pypi: None,
-    mock_legacy: None,
     source_type: None,
     pool: RepositoryPool,
 ) -> None:
@@ -403,19 +337,24 @@ def test_chooser_throws_an_error_if_package_hashes_do_not_match(
             package.version.text,
             source_type="legacy",
             source_reference="foo",
-            source_url="https://foo.bar/simple/",
+            source_url="https://legacy.foo.bar/simple/",
         )
 
     package.files = files
 
-    with pytest.raises(RuntimeError) as e:
+    with pytest.raises(PoetryRuntimeError) as e:
         chooser.choose_for(package)
-    assert files[0]["hash"] in str(e)
+
+    reason = f"Downloaded distributions for {package.name} ({package.version}) did not match any known checksums in your lock file."
+    assert str(e.value) == reason
+
+    text = e.value.get_text(debug=True, strip=True)
+    assert reason in text
+    assert files[0]["hash"] in text
 
 
-@pytest.mark.usefixtures("mock_legacy")
 def test_chooser_md5_remote_fallback_to_sha256_inline_calculation(
-    env: MockEnv, pool: RepositoryPool
+    env: MockEnv, pool: RepositoryPool, dist_hash_getter: DistributionHashGetter
 ) -> None:
     chooser = Chooser(pool, env)
     package = Package(
@@ -423,15 +362,14 @@ def test_chooser_md5_remote_fallback_to_sha256_inline_calculation(
         "0.1.0",
         source_type="legacy",
         source_reference="foo",
-        source_url="https://foo.bar/simple/",
+        source_url="https://legacy.foo.bar/simple/",
     )
     package.files = [
         {
-            "hash": (
-                "sha256:9fa123ad707a5c6c944743bf3e11a0e80d86cb518d3cf25320866ca3ef43e2ad"
-            ),
-            "filename": "demo-0.1.0.tar.gz",
+            "filename": filename,
+            "hash": (f"sha256:{dist_hash_getter(filename).sha256}"),
         }
+        for filename in [f"{package.name}-{package.version}.tar.gz"]
     ]
     res = chooser.choose_for(package)
     assert res.filename == "demo-0.1.0.tar.gz"
