@@ -5,7 +5,6 @@ import logging
 import re
 import uuid
 
-from dataclasses import dataclass
 from pathlib import Path
 from typing import TYPE_CHECKING
 from typing import Any
@@ -15,26 +14,23 @@ import pytest
 import requests
 
 from cleo.io.null_io import NullIO
+from keyring.credentials import SimpleCredential
 
+from poetry.console.exceptions import PoetryRuntimeError
 from poetry.utils.authenticator import Authenticator
 from poetry.utils.authenticator import RepositoryCertificateConfig
+from poetry.utils.password_manager import PoetryKeyring
 
 
 if TYPE_CHECKING:
-    from _pytest.logging import LogCaptureFixture
-    from _pytest.monkeypatch import MonkeyPatch
+    from pytest import LogCaptureFixture
+    from pytest import MonkeyPatch
     from pytest_mock import MockerFixture
 
     from tests.conftest import Config
     from tests.conftest import DummyBackend
 
 
-@dataclass
-class SimpleCredential:
-    username: str
-    password: str
-
-
 @pytest.fixture()
 def mock_remote(http: type[httpretty.httpretty]) -> None:
     http.register_uri(
@@ -101,14 +97,19 @@ def test_authenticator_ignores_locked_keyring(
     http: type[httpretty.httpretty],
     with_locked_keyring: None,
     caplog: LogCaptureFixture,
+    mocker: MockerFixture,
 ) -> None:
     caplog.set_level(logging.DEBUG, logger="poetry.utils.password_manager")
+    spy_get_credential = mocker.spy(PoetryKeyring, "get_credential")
+    spy_get_password = mocker.spy(PoetryKeyring, "get_password")
     authenticator = Authenticator()
     authenticator.request("get", "https://foo.bar/files/foo-0.1.0.tar.gz")
 
     request = http.last_request()
     assert request.headers["Authorization"] is None
-    assert "Keyring foo.bar is locked" in caplog.messages
+    assert "Accessing keyring failed during availability check" in caplog.messages
+    assert "Using keyring backend 'conftest LockedBackend'" in caplog.messages
+    assert spy_get_credential.call_count == spy_get_password.call_count == 0
 
 
 def test_authenticator_ignores_failing_keyring(
@@ -116,14 +117,20 @@ def test_authenticator_ignores_failing_keyring(
     http: type[httpretty.httpretty],
     with_erroneous_keyring: None,
     caplog: LogCaptureFixture,
+    mocker: MockerFixture,
 ) -> None:
     caplog.set_level(logging.DEBUG, logger="poetry.utils.password_manager")
+    spy_get_credential = mocker.spy(PoetryKeyring, "get_credential")
+    spy_get_password = mocker.spy(PoetryKeyring, "get_password")
     authenticator = Authenticator()
     authenticator.request("get", "https://foo.bar/files/foo-0.1.0.tar.gz")
 
     request = http.last_request()
     assert request.headers["Authorization"] is None
-    assert "Accessing keyring foo.bar failed" in caplog.messages
+
+    assert "Using keyring backend 'conftest ErroneousBackend'" in caplog.messages
+    assert "Accessing keyring failed during availability check" in caplog.messages
+    assert spy_get_credential.call_count == spy_get_password.call_count == 0
 
 
 def test_authenticator_uses_password_only_credentials(
@@ -159,7 +166,7 @@ def test_authenticator_uses_empty_strings_as_default_password(
     assert request.headers["Authorization"] == f"Basic {basic_auth}"
 
 
-def test_authenticator_uses_empty_strings_as_default_username(
+def test_authenticator_does_not_ignore_empty_strings_as_default_username(
     config: Config,
     mock_remote: None,
     repo: dict[str, dict[str, str]],
@@ -194,8 +201,9 @@ def test_authenticator_falls_back_to_keyring_url(
         }
     )
 
-    dummy_keyring.set_password(
-        "https://foo.bar/simple/", None, SimpleCredential("foo", "bar")
+    dummy_keyring.set_default_service_credential(
+        "https://foo.bar/simple/",
+        SimpleCredential("foo", "bar"),
     )
 
     authenticator = Authenticator(config, NullIO())
@@ -213,6 +221,7 @@ def test_authenticator_falls_back_to_keyring_netloc(
     http: type[httpretty.httpretty],
     with_simple_keyring: None,
     dummy_keyring: DummyBackend,
+    poetry_keyring: PoetryKeyring,
 ) -> None:
     config.merge(
         {
@@ -220,7 +229,10 @@ def test_authenticator_falls_back_to_keyring_netloc(
         }
     )
 
-    dummy_keyring.set_password("foo.bar", None, SimpleCredential("foo", "bar"))
+    dummy_keyring.set_default_service_credential(
+        "foo.bar",
+        SimpleCredential("foo", "bar"),
+    )
 
     authenticator = Authenticator(config, NullIO())
     authenticator.request("get", "https://foo.bar/files/foo-0.1.0.tar.gz")
@@ -268,9 +280,10 @@ def test_authenticator_request_raises_exception_when_attempts_exhausted(
     http.register_uri(httpretty.GET, sdist_uri, body=callback)
     authenticator = Authenticator(config, NullIO())
 
-    with pytest.raises(requests.exceptions.ConnectionError):
+    with pytest.raises(PoetryRuntimeError) as e:
         authenticator.request("get", sdist_uri)
 
+    assert str(e.value) == "All attempts to connect to foo.bar failed."
     assert sleep.call_count == 5
 
 
@@ -483,11 +496,13 @@ def test_authenticator_falls_back_to_keyring_url_matched_by_path(
         }
     )
 
-    dummy_keyring.set_password(
-        "https://foo.bar/alpha/files/simple/", None, SimpleCredential("foo", "bar")
+    dummy_keyring.set_default_service_credential(
+        "https://foo.bar/alpha/files/simple/",
+        SimpleCredential("foo", "bar"),
     )
-    dummy_keyring.set_password(
-        "https://foo.bar/beta/files/simple/", None, SimpleCredential("foo", "baz")
+    dummy_keyring.set_default_service_credential(
+        "https://foo.bar/beta/files/simple/",
+        SimpleCredential("foo", "baz"),
     )
 
     authenticator = Authenticator(config, NullIO())
